@@ -6,9 +6,10 @@ use crate::{
     cluster_slots::ClusterSlots,
     repair_weight::RepairWeight,
     result::Result,
-    serve_repair::{RepairType, ServeRepair, DEFAULT_NONCE},
+    serve_repair::{RepairType, ServeRepair, DEFAULT_NONCE, REPAIR_PEERS_CACHE_CAPACITY},
 };
 use crossbeam_channel::{Receiver as CrossbeamReceiver, Sender as CrossbeamSender};
+use lru::LruCache;
 use solana_ledger::{
     blockstore::{Blockstore, SlotMeta},
     shred::Nonce,
@@ -181,6 +182,7 @@ impl RepairService {
         let mut last_stats = Instant::now();
         let duplicate_slot_repair_statuses: HashMap<Slot, DuplicateSlotRepairStatus> =
             HashMap::new();
+        let mut peers_cache = LruCache::new(REPAIR_PEERS_CACHE_CAPACITY);
 
         loop {
             if exit.load(Ordering::Relaxed) {
@@ -259,13 +261,12 @@ impl RepairService {
                 )
             };
 
-            let mut cache = HashMap::new();
             let mut send_repairs_elapsed = Measure::start("send_repairs_elapsed");
             repairs.into_iter().for_each(|repair_request| {
                 if let Ok((to, req)) = serve_repair.repair_request(
                     &cluster_slots,
                     repair_request,
-                    &mut cache,
+                    &mut peers_cache,
                     &mut repair_stats,
                     &repair_info.repair_validators,
                 ) {
